@@ -39,13 +39,13 @@ lagm <- function(m, nLags) {
     }
   }
   if(ncol(m)>=2){
-    colnames(lagM)<-paste(colnames(m)[1:2],c(1:ncol(lagM)),sep = "_")
+    colnames(lagM)<-paste(colnames(m)[1:ncol(m)],rep(seq(nLags), each=ncol(m)),sep = "_")
   }
   else{
     colnames(lagM)<-paste(colnames(m),c(1:ncol(lagM)),sep = "_")
   }
   #seq(nLags*ncol(m))
-  return(lagM)
+  return(as.matrix(lagM))
 }
 
 ArchTest <- function (x, lags=12, demean = FALSE)
@@ -77,21 +77,45 @@ ArchTest <- function (x, lags=12, demean = FALSE)
 #'
 #'@param formula food~inf or food~inf|I(inf^2)
 #'@param data the dataframe
-#'@param p max lags of dependent variable
-#'@param q max lags of independent variables
+#'@param p  lags of dependent variable
+#'@param q  lags of independent variables
 #'@param ic : c("aic","bic","R2") criteria model selection
-#'@param graph 1 to show stability tests plot
+#'@param maxlags if TRUE auto lags selection
+#'@param graph TRUE to show stability tests plot
 #'@importFrom car linearHypothesis
-#'@importFrom stats lm AIC BIC as.formula model.frame model.matrix model.response na.omit sd update vcov embed resid
+#'@importFrom stats lm AIC BIC as.formula model.frame model.matrix model.response na.omit sd update vcov embed resid coef
 #'@importFrom graphics abline legend lines par plot
 #'@importFrom strucchange recresid
 #'@importFrom tseries jarque.bera.test
 #'@importFrom lmtest bgtest
 #'@import Formula
 #'@import matlab
-#'@import qpcR
+#'@examples
+#'
+#'############################################
+#'# Fit the nonlinear cointegrating autoregressive distributed lag model
+#'############################################
+#'# Load data
+#'data(fod)
+#'############################################
+#'#example 1: nardl with fixed p and q lags
+#'############################################
+#'reg<-nardl(food~inf,p=4,q=4,fod,ic="aic",maxlags = FALSE,graph = FALSE)
+#'summary(reg)
+#'
+#'############################################
+#'# example 2:auto selected lags (maxlags=TRUE)
+#'############################################
+#'reg<-nardl(food~inf,fod,ic="aic",maxlags = TRUE,graph = FALSE)
+#'summary(reg)
+#'
+#'############################################
+#'# example 3: Cusum and CusumQ plot (graph=TRUE)
+#'############################################
+#'reg<-nardl(food~inf,fod,ic="aic",maxlags = TRUE,graph = TRUE)
+#'
 #'@export
-nardl<-function(formula,data,p,q,ic=c("aic","bic","R2"),graph){
+nardl<-function(formula,data,p=NULL,q=NULL,ic=c("aic","bic","R2"),maxlags=TRUE,graph=FALSE){
   case<-3
   f<-formula
   a<-unlist(strsplit(as.character(f),"[|]"))
@@ -100,45 +124,59 @@ nardl<-function(formula,data,p,q,ic=c("aic","bic","R2"),graph){
     lhs   <- a[[2]]
     core  <- a[[3]]
     suffix <- a[[4]]
-    fm <- paste(lhs,"~",core,"|",suffix)
+    fm <-as.formula(paste(lhs,"~",core,"|",suffix))
     mf <- match.call(expand.dots = FALSE)
     m <- match(c("formula", "data", "subset", "na.action"), names(mf), 0)
     mf <- mf[c(1, m)]
-    ffm <- Formula::Formula(as.formula(fm))
+    ffm <- Formula::Formula(fm)
     mf[[1]] <- as.name("model.frame")
     mf$formula <- ffm
     fffm<-update(ffm, ~ . -1|~ . -1)
-    mf <- eval(mf, parent.frame())
+    mf <- model.frame(formula=fffm, data=data)
     x <- model.matrix(fffm, data = mf, rhs = 1)
+    if(ncol(x)>=2) stop("nardl package accept only one decomposed variable")
     h <- model.matrix(fffm, data = mf, rhs = 2)
-    y <- model.response(mf)
-    dy<-diff(as.matrix(y))
+    y<-model.response(mf)
+    y<-as.matrix(y)
+    #colnames(h)<-suffix
+    colnames(y)<-lhs
+    dy<-diff(y)
+    colnames(dy)<-paste("D",lhs,sep=".")
     dx<-diff(x)
+    colnames(dx)<-paste("D",colnames(x),sep=".")
     dh<-diff(h)
+    colnames(dh)<-paste("D",colnames(h),sep=".")
     n<-nrow(dx)
     cl<-ncol(dx)
     pos<-dx[,1:cl]>=0
-    dxp<-as.numeric(pos)*dx[,1:cl]
-    dxn<-(1-as.numeric(pos))*dx[,1:cl]
-    if(ncol(dx)>=2){
+    dxp<-as.matrix(as.numeric(pos)*dx[,1:cl])
+    colnames(dxp)<-paste(colnames(dx),"p",sep="_")
+    dxn<-as.matrix((1-as.numeric(pos))*dx[,1:cl])
+    colnames(dxn)<-paste(colnames(dx),"n",sep="_")
       xp<-apply(dxp,2,cumsum)
+      colnames(xp)<-paste(colnames(x),"p",sep="_")
       xn<-apply(dxn,2,cumsum)
-    }
-    else{
-      xp<-cumsum(dxp)
-      xn<-cumsum(dxn)
-    }
+      colnames(xn)<-paste(colnames(x),"n",sep="_")
 
     lagy<-lagm(as.matrix(y),1)
     lagh<-lagm(as.matrix(h),1)
     lxp<-lagm(as.matrix(xp),1)
     lxn<-lagm(as.matrix(xn),1)
     #fit<-lm(dy~na.omit(lagy)+lxp+lxn)
+
     #lag order selection
     R2 = NULL
     AK = NULL
     SC = NULL
-    ordmax<-max(p,q)
+
+    if(maxlags==TRUE){
+      ordmax<-floor(nrow(dx)^(1/3))/2
+    }
+    if(is.null(p) && is.null(q)){
+      ordmax<-floor(nrow(dx)^(1/3))/2
+    }else{ordmax<-max(p,q)}
+
+    #
     for (i in 1:ordmax){
       #lagmat = cbind(lagmat[-i,],x[(1):(l1-i)]) # lagged matrix
       # armod <- lm(x[(i+1):l1]~lagmat)
@@ -149,21 +187,30 @@ nardl<-function(formula,data,p,q,ic=c("aic","bic","R2"),graph){
       lagy<-na.omit(lagy)
       lagh<-na.omit(lagh)
       fit<-lm(dy~lagy+lxp+lxn+ldy+ldxp+ldxn+lagh+ldh)
-      R2[i] = qpcR::Rsq(fit)
+      R2[i] = summary(fit)$adj.r.squared
       AK[i] = AIC(fit)
       SC[i] = BIC(fit)
     }
     np<-0
+    #ores<-c(which.max(R2), which.min(AK),which.min(SC) )
     if(ic=="aic") np<-which.min(AK)
     if(ic=="bic") np<-which.min(SC)
     if(ic=="R2") np<-which.min(R2)
+    #nnp<-ores[duplicated(ores)]
+    #nnp<-nnp[1]
+    #np<-3
     ldy<-lagm(dy,c(1:np))
     ldh<-lagm(as.matrix(dh),c(1:np))
     ldxp<-lagm(as.matrix(dxp),c(1:np))
     ldxn<-lagm(as.matrix(dxn),c(1:np))
     lagy<-na.omit(lagy)
     lagh<-na.omit(lagh)
+    lhnames<-colnames(dy)
+    rhnams<-c("Const",colnames(lagy),colnames(lxp),colnames(lxn),
+              colnames(ldy),colnames(ldxp),colnames(ldxn),
+              colnames(lagh),colnames(ldh))
     fit<-lm(dy~lagy+lxp+lxn+ldy+ldxp+ldxn+lagh+ldh)
+    names(fit$coefficients)<-rhnams
     fitcoef<-summary(fit)
 
   }else{
@@ -174,23 +221,26 @@ nardl<-function(formula,data,p,q,ic=c("aic","bic","R2"),graph){
     fffm<-update(ffm, ~ . -1)
     mf <- model.frame(formula=fffm, data=data)
     x <- model.matrix(attr(mf, "terms"), data=mf)
+    if(ncol(x)>=2) stop("nardl package accept only one decomposed variable")
     y <- model.response(mf)
-    dy<-diff(as.matrix(y))
+    y <- model.response(mf)
+    y<-as.matrix(y)
+    colnames(y)<-lhs
+    dy<-diff(y)
+    colnames(dy)<-paste("D",lhs,sep=".")
     dx<-diff(x)
+    colnames(dx)<-paste("D",core,sep=".")
     n<-nrow(dx)
     cl<-ncol(dx)
     pos<-dx[,1:cl]>=0
-    dxp<-as.numeric(pos)*dx[,1:cl]
-    dxn<-(1-as.numeric(pos))*dx[,1:cl]
-    if(ncol(dx)>=2){
+    dxp<-as.matrix(as.numeric(pos)*dx[,1:cl])
+    colnames(dxp)<-paste(colnames(dx),"p",sep="_")
+    dxn<-as.matrix((1-as.numeric(pos))*dx[,1:cl])
+    colnames(dxn)<-paste(colnames(dx),"n",sep="_")
       xp<-apply(dxp,2,cumsum)
+      colnames(xp)<-paste(colnames(x),"p",sep="_")
       xn<-apply(dxn,2,cumsum)
-    }
-    else{
-      xp<-cumsum(dxp)
-      xn<-cumsum(dxn)
-    }
-
+      colnames(xn)<-paste(colnames(x),"n",sep="_")
     lagy<-lagm(as.matrix(y),1)
     lxp<-lagm(as.matrix(xp),1)
     lxn<-lagm(as.matrix(xn),1)
@@ -198,14 +248,17 @@ nardl<-function(formula,data,p,q,ic=c("aic","bic","R2"),graph){
     R2 = NULL
     AK = NULL
     SC = NULL
-    ordmax<-max(p,q)
+    if(maxlags==TRUE){
+      ordmax<-floor(nrow(dx)^(1/3))/2
+    }
+    else{ordmax<-max(p,q)}
     for (i in 1:ordmax){
       ldy<-lagm(dy,c(1:i))
       ldxp<-lagm(as.matrix(dxp),c(1:i))
       ldxn<-lagm(as.matrix(dxn),c(1:i))
       lagy<-na.omit(lagy)
       fit<-lm(dy~lagy+lxp+lxn+ldy+ldxp+ldxn)
-      R2[i] = qpcR::Rsq(fit)
+      R2[i] = summary(fit)$adj.r.squared
       AK[i] = AIC(fit)
       SC[i] = BIC(fit)
     }
@@ -217,7 +270,12 @@ nardl<-function(formula,data,p,q,ic=c("aic","bic","R2"),graph){
     ldxp<-lagm(as.matrix(dxp),c(1:np))
     ldxn<-lagm(as.matrix(dxn),c(1:np))
     lagy<-na.omit(lagy)
+    rhnams<-c("Const",colnames(lagy),colnames(lxp),colnames(lxn),
+              colnames(ldy),colnames(ldxp),colnames(ldxn))
+
+    # flm<-paste(lhnames,"~",paste(rhnams,collapse = " + "))
     fit<-lm(dy~lagy+lxp+lxn+ldy+ldxp+ldxn)
+    names(fit$coefficients)<-rhnams
     fitcoef<-summary(fit)
   }
 
@@ -264,18 +322,18 @@ nardl<-function(formula,data,p,q,ic=c("aic","bic","R2"),graph){
   #get recursive residuals
   rece<-strucchange::recresid(fit)
  # plot cumsum and cumsumq
-  if(graph==1){
+  if(graph==TRUE){
     bbst<-sel$coefficients[,1]
     kt<-length(bbst[-1])
-    n<-length(rece)
-    cusum(rece,kt)
+    n<-length(residu)
+    cusum(rece,kt,n)
     cumsq(rece,kt,n)
   }
 
  out<-list(fstat=fstat, fit=fit,fitcoef=fitcoef, sel=sel, cof=cof,coof=coof,
        k=k,case=case,lvars=lvars,vcc=vcc,fb=fb,fb1=fb1,fb2=fb2,
        lrse=lrse,lrt=lrt,lrpv=lrpv,lres=lres,selresidu=residu,
-       tasym=tasym,pasym=pasym,jbtest=jbtest,lmtest=lmtest,arch=arch,np=np)
+       tasym=tasym,pasym=pasym,jbtest=jbtest,lmtest=lmtest,arch=arch,np=np,rece=rece)
  class(out) <- "nardl"
  # Return results.
  return(out)
@@ -306,14 +364,14 @@ cumsq<-function(e,k,n){
          bty = "n",
          c("CUSUM of squares","5% significance"),
          lty = c(1, 1),
-         cex=0.8,
+         cex=0.6,
          col=c("blue","red") )
 
 }
 
-cusum<-function(e,k){
+cusum<-function(e,k,n){
   w<-as.matrix(na.omit(e))
-  n<-length(e)
+  #n<-length(e)
   w=cumsum(w/apply(w, 2, sd))
   c=0.984
   w2=seqa(c*sqrt(n-k),(2*c*sqrt(n-k))/length(w),length(w))
@@ -330,10 +388,10 @@ cusum<-function(e,k){
   abline(h=0,lty=2)
   legend(par("usr")[2],par("usr")[4],
          xpd = TRUE ,
-        bty = "n",
+         bty = "n",
          c("CUSUM ","5% significance"),
          lty = c(1, 1),
-         cex=0.8,
+         cex=0.6,
          col=c("blue","red") )
 
 }
@@ -346,6 +404,7 @@ seqa<-function(a,b,c){
 
 
 
+
 #' summary
 #'
 #' @param object is the object of the function
